/***
@author 8boomgar & 8njemz
@date	2010/02/23 16:20
@descr  the main prolog file for server code
*/
:-retractall(wl(_)).
:-consult('servers_little_helpers.pl').
:-consult('predicate_whitelist.pl').
:-dynamic(client/5).
:-dynamic(id2nick/2).
:-retractall(client(_,_,_,_,_)).
:-retractall(id2nick(_,_)).

% client(PIP,PPort,MasterSocket,NewSocket,ID)

/**
@form	start_server
@descr	starts server with standard port
*/
start_server:-
	portnumber(P),
	start_server(P).

/**
@form	start_server(+Port)
@constraints 
@ground Port
@descr	starts server with given port

*/
start_server(P):-
	new(@s, socket(P)),
	send(@s, accept_message, message(@prolog, accepted_client, @receiver, @arg1)),
	send(@s, input_message, message(@prolog, receive_msg, @arg1)),
	send(@s, listen),
	%get_peer(@s,IP, MPort),
	get(@s, address, IP),
	
	get_ip(Myip),
	concat('IP is: ', Myip, Txt1),
	concat(Txt1, ', Port is: ', Txt2),
	concat(Txt2, IP, Txt3),
	( write_log(Txt3) ; format('LIP is: ~w, LPort is: ~w~n', [IP,'port'])).

	 
/**
@form   exit/0
@descr	close server socket, destroy socket object
*/
exit:-
	send(@s,close), free(@s),
	retractall(client(_,_,_,_,_)),
	write_log('server closed').


/**
@form   accepted_client(+MasterSocket, +NewSocket)
@descr  executed whenever the server establishes a connection to a client
*/
accepted_client(MasterSocket, NewSocket) :-
	get(MasterSocket, address, MPort),
	get(NewSocket, address, NPort),
	get_peer(NewSocket,PIP,PPort),
	client_acception_procedure(
	MasterSocket,NewSocket, MPort, NPort, PPort, PIP
        ).


/**
@form	receive_msg(+Message) 
@constraints
@ground	Message
@descr  handles the <i>Message</i>s received from clients
*/
receive_msg(Message) :-
	get(Message, value, Atom),
	atom_to_term(Atom, connection_request(ID), _),
	client(PIP,PPort,MasterSocket,NewSocket,ID),
	get(MasterSocket, address, MPort),
	get(NewSocket, address, NPort),
	% add MPort and NPort
	client_acception_procedure(
	MasterSocket,NewSocket,MPort,NPort,PPort,PIP
        ).
	%highest_ID(N),
	%N1 is N + 1,
	%assert(	client(PIP,PPort,MasterSocket,NewSocket,N1)).
receive_msg(Message) :-
	get(Message, value, Atom),
	atom_to_term(Atom, disconnect_request(ID), _),
	id2socket(ID, Socket),
	write_ln('in unlink procedure'),
	send(Socket, unlink),
	leave_message(ID),
	( retractall(id2nick(ID,_)) ; true ),
	retractall(client(_,_,_,Socket,_)).
receive_msg(Message) :-
	get(Message, value, Atom),
	atom_to_term(Atom, chat(X, Text), _),
	(   id2nick(X,Nick) ; Nick = X),
	term_to_atom(append_to_history(chat(Nick,Text)),Atom2),
	send_to_all(Atom2), % chat is sent to all clients
%	id2nick(X,Nick),
	concat(Nick, ': ', Txt1),
	concat(Txt1, Text, Txt2),
	( write_log(Txt2) ; format('~w: ~w~n', [Nick,Text])).
receive_msg(Message) :-
	get(Message, value, Atom),
	atom_to_term(Atom, nick_chosen(ID,Nick), _),
	nick_change(nick_chosen(ID,Nick)).
receive_msg(Message) :-
	get(Message, value, Atom),
	atom_to_term(Atom, Term, _),
	Term =.. [N|L],
	(   wl(N) ; 
	    (concat('pred: ', N, P),
	    concat(P, ' not in whitelist', Txt),
	    write_log(Txt))
	),
	!,
	L= [Color|Output],
	format('sending event: ~w~n', [Atom]), % debug output
	%color2id(Color,ID),
	event(Color, N, Output).



%receive_msg(Message) :-
%	get(Message, value, Atom),
%	
%	execute_pred(Pred,Vars,Result),
%	send_to_all(Result).
	


/**
@form   send_to_all(+Message)
@constraints
@ground Message
@descr  send given <i>Message</i> to all known clients
*/
send_to_all(Message) :-
        %findall(Client, client(_,_,_,Client,_), Clients),
        get(@s, clients, Chain),
	chain_list(Chain,Clients),
	send_to(Clients,Message).


send_to(Clients,Message) :- send_to(Clients,[],Message).
/**
@form   send_to(+ListOfClients, +Message)
@constraints 
@ground ListOfClients
@ground Message
@descr	sends <i>Message</i> to <i>ListOfClients</i>
*/
send_to([],_,_) :- !.
send_to([C|Clients],Sent,Message) :-
	\+ member(C, Sent),
	send(C,	append_line, Message),
	send_to(Clients,[C|Sent],Message).
send_to([C|Clients],Sent,Message) :-
	member(C,Sent),
	send_to(Clients,Sent,Message).

/**
@form   send_to_socket(+Socket, +Message)
@constraints
@ground	Socket
@ground	Message
@descr	sends <i>Message</i> to specified <i>Socket</i>
*/
send_to_socket(Socket, Message) :-
	send(Socket, append_line, Message).












